
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Float_Text_IO; use Ada.Text_IO;
with Ada.Task_Identification; use Ada.Task_Identification;
with Ada.Calendar.Formatting; use Ada.Calendar.Formatting;
with Ada.Real_Time; use Ada.Real_Time;
with A4A.Sigint_Handler; use A4A.Sigint_Handler;


package body A4A.Kernel is

   task body Main_Loop is
      My_ID : Task_Id;
      Start_Time : constant Ada.Calendar.Time := Ada.Calendar.Clock;
   begin
      My_ID := Current_Task;
      Put_Line ("Main_Loop's ID is " & Image (My_ID));
      Put_Line ("Started at " & Ada.Calendar.Formatting.Image (Date => Start_Time,
                                                               Include_Time_Fraction => True,
                                                               Time_Zone => 0));
      My_DPM.Outputs_Write(Outputs => A4A.Application.MyOutputs);
      loop
         if My_DPM.New_Inputs then
            My_DPM.Get_Inputs(Inputs => A4A.Application.MyInputs);
         end if;

         A4A.Application.Main_Cyclic;

         My_DPM.Outputs_Write(Outputs => A4A.Application.MyOutputs);
         exit when Quit;
         delay 1.0;
      end loop;
      Put_Line ("Main_Loop finished !");
   end Main_Loop;

   task body Sig_Handler is
   begin
      Put_Line ("Waiting Signal Interrupt... Use Ctrl+C to exit");
      Handler.Wait;
      Quit := True;
      Put_Line ("Signal Interrupt caught !");
   end Sig_Handler;

   task body Periodic_Task is
      My_ID : Task_Id;
      Next_Time : Ada.Real_Time.Time := Clock;
      MyPeriod : constant Time_Span := Milliseconds (Period);
      MyTick_Duration : Duration := To_Duration (Tick);
      MyDuration : Duration;
      MyMinDuration : Duration;
      MyMaxDuration : Duration;
      MyPeriodDuration : Duration := To_Duration (MyPeriod);
      Max_Times : Integer := 10000/Period;
      Times : Integer := 0;
      Stats : array (1 .. 10) of Integer;
      Thresholds : array (1 .. 10) of Duration;
      MyInputs  : A4A.Application.Inputs_Type  := (others => 0);
      MyOutputs : A4A.Application.Outputs_Type := (others => 0);
   begin
      My_ID := Current_Task;

      Put_Line ("Periodic_Task'S ID is " & Image (My_ID));
      Put_Line ("Millisecond Duration : " & Duration'Image (To_Duration (Milliseconds (1))));
      Put_Line ("Microsecond Duration : " & Duration'Image (To_Duration (Microseconds (1))));
      Put_Line ("Nanosecond Duration  : " & Duration'Image (To_Duration (Nanoseconds  (1))));
      Put_Line ("MyTick_Duration      : " & Duration'Image (MyTick_Duration));
      Put_Line ("MyPeriod_Duration    : " & Duration'Image (MyPeriodDuration));

      for Index in Stats'Range loop
         Stats (Index) := 0;
      end loop;

      Thresholds( 1) := MyPeriodDuration *  0.001;
      Thresholds( 2) := MyPeriodDuration *  0.005;
      Thresholds( 3) := MyPeriodDuration *  0.010;
      Thresholds( 4) := MyPeriodDuration *  0.050;
      Thresholds( 5) := MyPeriodDuration *  0.100;
      Thresholds( 6) := MyPeriodDuration *  0.500;
      Thresholds( 7) := MyPeriodDuration *  1.000;
      Thresholds( 8) := MyPeriodDuration *  5.000;
      Thresholds( 9) := MyPeriodDuration * 10.000;
      Thresholds(10) := MyPeriodDuration * 20.000;

      loop
         Next_Time := Next_Time + MyPeriod;

         -- Do something useful here

         if My_DPM.New_Outputs then
            My_DPM.Get_Outputs(Outputs => MyOutputs);
            MyInputs(1) := MyOutputs(1);
            My_DPM.Inputs_Write(Inputs => MyInputs);
         end if;

         delay until Next_Time;

         MyDuration := To_Duration (Clock - Next_Time);
         if (Times = 0) then
            MyMinDuration := MyDuration;
            MyMaxDuration := MyDuration;
         else
            if MyMinDuration > MyDuration then
               MyMinDuration := MyDuration;
            else if MyMaxDuration < MyDuration then
                  MyMaxDuration := MyDuration;
               end if;
            end if;
         end if;

         for Index in Stats'Range loop
            if MyDuration < Thresholds (Index) then
               Stats (Index) := Stats (Index) + 1;
               exit;
            end if;
         end loop;

         Times := Times + 1;

         exit when Times > Max_Times;
         exit when Quit;

      end loop;

      if Quit then
         Put_Line ("Periodic_Task told to quit !");
      end if;

      Put_Line ("Et Hop ! Times : " & Times'Img & " Duration Min : " & Duration'Image (MyMinDuration) & " Max : " & Duration'Image (MyMaxDuration));
      for Index in Stats'Range loop
         Put_Line (Integer'Image(Index) & " | Thresholds := " & Duration'Image(Thresholds(Index)) & " | Stats := " & Integer'Image(Stats(Index)));
      end loop;
      Put_Line ("Periodic_Task finished !");
   end Periodic_Task;

   procedure Run_1 is
   begin
      if My_DPM.New_Inputs then
         My_DPM.Get_Inputs(Inputs => A4A.Application.MyInputs);
      end if;

      A4A.Application.Periodic_Run_1;

      My_DPM.Outputs_Write(Outputs => A4A.Application.MyOutputs);

   end Run_1;
end A4A.Kernel;
