
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

with System; use System;
with GNAT.OS_Lib; -- GNAT.OS_Lib.Errno
-- with System.OS_Interface; -- System.OS_Interface.errno

with Interfaces.C;use Interfaces.C;
with Interfaces.C.Strings;

package body A4A.Protocols.LibModbus is

   function Modbus_Strerror
     return String is
      function C_Modbus_Strerror (errnum : in Interfaces.C.int)
                                return Interfaces.C.Strings.chars_ptr;
      --const char *modbus_strerror(int errnum);
      pragma Import (C, C_Modbus_Strerror, "modbus_strerror");
   begin
      return Interfaces.C.Strings.Value (C_Modbus_Strerror (Interfaces.C.int(GNAT.OS_Lib.Errno)));
   end Modbus_Strerror;

   procedure Set_Debug (Context : in Context_Type;
                        On      : in Boolean) is
      procedure Modbus_Set_Debug (Context : in Context_Type;
                                  On      : in Interfaces.C.int);
      --void modbus_set_debug(modbus_t *ctx, int boolean);
      pragma Import (C, Modbus_Set_Debug, "modbus_set_debug");
      Debug_On : Interfaces.C.int;
   begin
      if On then
         Debug_On := 1;
      else
         Debug_On := 0;
      end if;
      Modbus_Set_Debug(Context => Context, On => Debug_On);
   end Set_Debug;

   function New_TCP (IP_Address : in String;
                     Port       : in TCP_Port_Type)
                     return Context_Type is
      function Modbus_New_TCP (IP_Address : in Interfaces.C.Strings.chars_ptr;
                               Port       : in Interfaces.C.int)
                               return System.Address;
      -- modbus_t* modbus_new_tcp(const char *ip_address, int port);
      pragma Import (C, Modbus_New_TCP, "modbus_new_tcp");
      IP_Address_ptr : Interfaces.C.Strings.chars_ptr;
      Result         : System.Address;
   begin
      IP_Address_ptr := Interfaces.C.Strings.New_String (IP_Address);
      Result := Modbus_New_TCP(IP_Address => IP_Address_ptr,
                               Port       => Interfaces.C.int(Port));
      Interfaces.C.Strings.Free(IP_Address_ptr);
      if Result = System.Null_Address then
         raise Context_Error with Modbus_Strerror;
      end if;

      return Context_Type(Result);
   end New_TCP;

   procedure Connect (Context : in Context_Type) is
      function Modbus_Connect (Context : in Context_Type)
                               return Interfaces.C.int;
      -- int modbus_connect(modbus_t *ctx);
      pragma Import (C, Modbus_Connect, "modbus_connect");
   begin
      if Modbus_Connect (Context) = -1 then
         raise Connect_Error with Modbus_Strerror;
      end if;
   end Connect;

   procedure Read_Registers
     (Context     : in Context_Type;
      Offset      : in Offset_Type;
      Number      : in Read_Register_Quantity_Type;
      Dest        : in System.Address) is
      function Modbus_Read_Registers
        (Context     : in Context_Type;
         Offset      : in Interfaces.C.int;
         Number      : in Interfaces.C.int;
         Dest        : in System.Address)
         return Interfaces.C.int;
      --int modbus_read_registers(modbus_t *ctx, int addr, int nb, uint16_t *dest);
      pragma Import (C, Modbus_Read_Registers, "modbus_read_registers");
   begin
      if Modbus_Read_Registers (Context,
                                Interfaces.C.int(Offset),
                                Interfaces.C.int(Number),
                                Dest) = -1 then
         raise Other_Error with Modbus_Strerror;
      end if;
   end Read_Registers;

   procedure Read_Input_Registers
     (Context     : in Context_Type;
      Offset      : in Offset_Type;
      Number      : in Read_Register_Quantity_Type;
      Dest        : in System.Address) is
      function Modbus_Read_Input_Registers
        (Context     : in Context_Type;
         Offset      : in Interfaces.C.int;
         Number      : in Interfaces.C.int;
         Dest        : in System.Address)
         return Interfaces.C.int;
      --int modbus_read_input_registers(modbus_t *ctx, int addr, int nb, uint16_t *dest);
      pragma Import (C, Modbus_Read_Input_Registers, "modbus_read_input_registers");
   begin
      if Modbus_Read_Input_Registers (Context,
                                      Interfaces.C.int(Offset),
                                      Interfaces.C.int(Number),
                                      Dest) = -1 then
         raise Other_Error with Modbus_Strerror;
      end if;
   end Read_Input_Registers;

   procedure Write_Registers
     (Context      : in Context_Type;
      Offset       : in Offset_Type;
      Number       : in Write_Register_Quantity_Type;
      Data         : in System.Address) is
      function Modbus_Write_Registers
        (
         Context     : in Context_Type;
         Offset      : in Interfaces.C.int;
         Number      : in Interfaces.C.int;
         Data        : in System.Address)
         return Interfaces.C.int;
      -- int modbus_write_registers(modbus_t *ctx, int addr, int nb, const uint16_t *data);
      pragma Import (C, Modbus_Write_Registers, "modbus_write_registers");
   begin
      if Modbus_Write_Registers (Context,
                                 Interfaces.C.int(Offset),
                                 Interfaces.C.int(Number),
                                 Data) = -1 then
         raise Other_Error with Modbus_Strerror;
      end if;
   end Write_Registers;

   function Mapping_New
     (Nb_Coil_Status       : in Interfaces.C.int;
      Nb_Input_Status      : in Interfaces.C.int;
      Nb_Holding_Registers : in Interfaces.C.int;
      Nb_Input_Registers   : in Interfaces.C.int)
      return System.Address is
      function Modbus_Mapping_New
        (Nb_Coil_Status       : in Interfaces.C.int;
         Nb_Input_Status      : in Interfaces.C.int;
         Nb_Holding_Registers : in Interfaces.C.int;
         Nb_Input_Registers   : in Interfaces.C.int)
         return System.Address;
      --modbus_mapping_t* modbus_mapping_new(int nb_coil_status, int nb_input_status,
      --                                  int nb_holding_registers, int nb_input_registers);
      pragma Import (C, Modbus_Mapping_New, "modbus_mapping_new");
      Result         : System.Address;
   begin
      Result := Modbus_Mapping_New(Nb_Coil_Status,
                                   Nb_Input_Status,
                                   Nb_Holding_Registers,
                                   Nb_Input_Registers);
      if Result = System.Null_Address then
         raise Other_Error with Modbus_Strerror;
      end if;

      return Result;
   end Mapping_New;

   function Mapping_New2
     (Nb_Coil_Status       : in Interfaces.C.int;
      Nb_Input_Status      : in Interfaces.C.int;
      Nb_Holding_Registers : in Interfaces.C.int;
      Nb_Input_Registers   : in Interfaces.C.int)
      return Modbus_Mapping_Type2 is
      function Modbus_Mapping_New
        (Nb_Coil_Status       : in Interfaces.C.int;
         Nb_Input_Status      : in Interfaces.C.int;
         Nb_Holding_Registers : in Interfaces.C.int;
         Nb_Input_Registers   : in Interfaces.C.int)
         return System.Address;
      --modbus_mapping_t* modbus_mapping_new(int nb_coil_status, int nb_input_status,
      --                                  int nb_holding_registers, int nb_input_registers);
      pragma Import (C, Modbus_Mapping_New, "modbus_mapping_new");
      Result         : System.Address;
   begin
      Result := Modbus_Mapping_New(Nb_Coil_Status,
                                   Nb_Input_Status,
                                   Nb_Holding_Registers,
                                   Nb_Input_Registers);
      if Result = System.Null_Address then
         raise Other_Error with Modbus_Strerror;
      end if;

      declare
         My_C_Mapping : Modbus_Mapping_Type;
         for My_C_Mapping'Address use Result;

         MyMapping : Modbus_Mapping_Type2;

--           My_tab_input_registers : aliased Word_Array(0..Integer(Nb_Input_Registers));
--           for My_tab_input_registers'Address use My_C_Mapping.tab_input_registers;
--
--           My_tab_registers : aliased Word_Array(0..Integer(Nb_Holding_Registers));
--           for My_tab_registers'Address use My_C_Mapping.tab_registers;
      begin
         MyMapping.C_Mapping_Address := Result;
--           MyMapping.tab_input_registers := My_tab_input_registers'Access;
--           MyMapping.tab_registers := My_tab_registers'Access;

         return MyMapping;
      end;
   end Mapping_New2;

   function TCP_Listen (Context       : in Context_Type;
                        Nb_Connection : in Integer)
                        return Socket_Type is
      function Modbus_TCP_Listen (Context       : in Context_Type;
                                  Nb_Connection : in Interfaces.C.int)
                                  return Socket_Type;
      --int modbus_tcp_listen(modbus_t *ctx, int nb_connection);
      pragma Import (C, Modbus_TCP_Listen, "modbus_tcp_listen");
      Result : Socket_Type;
   begin
      Result := Modbus_TCP_Listen (Context,
                                   Interfaces.C.int(Nb_Connection));
      if Result = No_Socket then
         raise Other_Error with Modbus_Strerror;
      end if;

      return Result;
   end TCP_Listen;

   procedure TCP_Accept (Context       : in Context_Type;
                         Socket_Access : in System.Address) is
      function Modbus_TCP_Accept (Context       : in Context_Type;
                                  Socket_Access : in System.Address)
                                  return Interfaces.C.int;
      --int modbus_tcp_accept(modbus_t *ctx, int *socket);
      pragma Import (C, Modbus_TCP_Accept, "modbus_tcp_accept");
      Result : Interfaces.C.int;
   begin
      Result := Modbus_TCP_Accept (Context,
                                   Socket_Access);
      if Result = -1 then
         raise Other_Error with Modbus_Strerror;
      end if;
   end TCP_Accept;

end A4A.Protocols.LibModbus;
