
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

-- <summary>
-- This package provides the binding to libmodbus, a library implementing
-- the modbus RTU and TCP protocols.
-- </summary>
-- <description>
-- It provides :
-- - libmodbus data types,
-- - libmodbus C functions binding.
-- </description>
-- <group>Protocols</group>
-- <c_version>3.0.1</c_version>

with System;
with Interfaces.C.Strings;

with GNAT.Sockets; use GNAT.Sockets;

package A4A.Protocols.LibModbus is

   Context_Error : exception;
   Connect_Error : exception;
   Other_Error   : exception;

   -- from modbus.h

   MODBUS_MAX_READ_REGISTERS     : constant := 125;
   --  #define MODBUS_MAX_READ_REGISTERS          125
   MODBUS_MAX_WRITE_REGISTERS    : constant := 123;
   --  #define MODBUS_MAX_WRITE_REGISTERS         123
   MODBUS_MAX_RW_WRITE_REGISTERS : constant := 121;
   --  #define MODBUS_MAX_RW_WRITE_REGISTERS      121

   subtype Offset_Type is Integer range 0..65535;
   subtype Read_Register_Quantity_Type is
     Integer range 0..MODBUS_MAX_READ_REGISTERS;
   subtype Write_Register_Quantity_Type is
     Integer range 0..MODBUS_MAX_WRITE_REGISTERS;
   subtype Read_Write_Register_Quantity_Type is
     Integer range 0..MODBUS_MAX_RW_WRITE_REGISTERS;

   type Context_Type is private;
   -- Modbus context : RTU or TCP

   type Modbus_Mapping_Type is
      record
         nb_bits             : Interfaces.C.int;
         nb_input_bits       : Interfaces.C.int;
         nb_input_registers  : Interfaces.C.int;
         nb_registers        : Interfaces.C.int;
         tab_bits            : System.Address;
         tab_input_bits      : System.Address;
         tab_input_registers : System.Address;
         tab_registers       : System.Address;
      end record;
   --     typedef struct {
   --       int nb_bits;
   --       int nb_input_bits;
   --       int nb_input_registers;
   --       int nb_registers;
   --       uint8_t *tab_bits;
   --       uint8_t *tab_input_bits;
   --       uint16_t *tab_input_registers;
   --       uint16_t *tab_registers;
   --     } modbus_mapping_t;
   type Modbus_Mapping_Type_Access is access all Modbus_Mapping_Type;

   type Modbus_Mapping_Type2 is
      record
         tab_input_registers : Word_Array_Access;
         tab_registers       : Word_Array_Access;
         C_Mapping_Address   : System.Address; -- needed for freeing
      end record;
   type Modbus_Mapping_Type2_Access is access all Modbus_Mapping_Type2;

   procedure Connect (Context : in Context_Type);
   -- May raise an Error
   -- int modbus_connect(modbus_t *ctx);

   procedure Close (Context : in Context_Type);
   --void modbus_close(modbus_t *ctx);
   pragma Import (C, Close, "modbus_close");

   procedure Free (Context : in Context_Type);
   --void modbus_free(modbus_t *ctx);
   pragma Import (C, Free, "modbus_free");

   procedure Set_Debug (Context : in Context_Type;
                        On      : in Boolean);
   --void modbus_set_debug(modbus_t *ctx, int boolean);

   procedure Read_Registers
     (Context     : in Context_Type;
      Offset      : in Offset_Type;
      Number      : in Read_Register_Quantity_Type;
      Dest        : in System.Address);
   -- May raise an Error
   --int modbus_read_registers(modbus_t *ctx, int addr, int nb, uint16_t *dest);

   procedure Read_Input_Registers
     (Context     : in Context_Type;
      Offset      : in Offset_Type;
      Number      : in Read_Register_Quantity_Type;
      Dest        : in System.Address);
   -- May raise an Error
   --int modbus_read_input_registers(modbus_t *ctx, int addr, int nb, uint16_t *dest);

   procedure Write_Registers
     (Context      : in Context_Type;
      Offset       : in Offset_Type;
      Number       : in Write_Register_Quantity_Type;
      Data         : in System.Address);
   -- May raise an Error
   -- int modbus_write_registers(modbus_t *ctx, int addr, int nb, const uint16_t *data);

   function Mapping_New2
     (Nb_Coil_Status       : in Interfaces.C.int;
      Nb_Input_Status      : in Interfaces.C.int;
      Nb_Holding_Registers : in Interfaces.C.int;
      Nb_Input_Registers   : in Interfaces.C.int)
      return Modbus_Mapping_Type2;

   function Mapping_New
     (Nb_Coil_Status       : in Interfaces.C.int;
      Nb_Input_Status      : in Interfaces.C.int;
      Nb_Holding_Registers : in Interfaces.C.int;
      Nb_Input_Registers   : in Interfaces.C.int)
      return System.Address;
   -- May raise an Error
   --modbus_mapping_t* modbus_mapping_new(int nb_coil_status, int nb_input_status,
   --                                  int nb_holding_registers, int nb_input_registers);

   procedure Mapping_Free (Mapping_Access : in System.Address);
   --void modbus_mapping_free(modbus_mapping_t *mb_mapping);
   pragma Import (C, Mapping_Free, "modbus_mapping_free");

   function Receive (Context     : in Context_Type;
                     Request     : in System.Address) return Interfaces.C.int;
   --int modbus_receive(modbus_t *ctx, uint8_t *req);
   pragma Import (C, Receive, "modbus_receive");

   function Reply (Context        : in Context_Type;
                   Request        : in System.Address;
                   Request_Len    : in Interfaces.C.int;
                   Mapping_Access : in System.Address) return Interfaces.C.int;
   --int modbus_reply(modbus_t *ctx, const uint8_t *req,
   --              int req_length, modbus_mapping_t *mb_mapping);
   pragma Import (C, Reply, "modbus_reply");

   subtype TCP_Port_Type is Integer range 0..65535;

   -- from modbus-tcp.h
   MODBUS_TCP_MAX_ADU_LENGTH  : constant := 260;

   function New_TCP (IP_Address : in String;
                     Port       : in TCP_Port_Type) return Context_Type;
   -- May raise an Error
   -- modbus_t* modbus_new_tcp(const char *ip_address, int port);

   subtype Socket_Type is GNAT.Sockets.Socket_Type;
--     type Socket_Type_Access is access all Socket_Type;
--     pragma Convention (C, Socket_Type_Access);

   function TCP_Listen (Context       : in Context_Type;
                        Nb_Connection : in Integer) return Socket_Type;
   -- May raise an Error
   --int modbus_tcp_listen(modbus_t *ctx, int nb_connection);

   procedure TCP_Accept (Context       : in Context_Type;
                         Socket_Access : in System.Address);
   -- May raise an Error
   --int modbus_tcp_accept(modbus_t *ctx, int *socket);

   procedure Set_Socket (Context       : in Context_Type;
                         Socket        : in Socket_Type);
   --void modbus_set_socket(modbus_t *ctx, int socket);
   pragma Import (C, Set_Socket, "modbus_set_socket");

   function Get_Socket (Context       : in Context_Type) return Socket_Type;
   --int modbus_get_socket(modbus_t *ctx);
   pragma Import (C, Get_Socket, "modbus_get_socket");

private
   type Context_Type is new System.Address;

   function Modbus_Strerror return String;
   --const char *modbus_strerror(int errnum);

end A4A.Protocols.LibModbus;
