
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

-- <summary>
-- Ada for Automation is a framework to build industrial control
-- applications using the Ada language.
-- It provides :
-- - tasks, cyclic and periodic ones,
-- - libraries, containing common functions and function blocs and
-- components,
-- - automation field protocols,
-- - add more by yourself.
-- </summary>
-- <description>
-- Let's define some IEC 61131-3 types so that they are available
-- to any child package, including user's ones, as well as some
-- elementary functions.
-- Since most types are already defined in package Interfaces,
-- let's rename them. Elementary functions as well.
-- type Boolean already exists in Ada.
-- </description>

with Interfaces;
package A4A is

   type Byte is new Interfaces.Unsigned_8;
   --  8-bit unsigned integer

   type Word is new Interfaces.Unsigned_16;
   -- 16-bit unsigned integer

   type DWord is new Interfaces.Unsigned_32;
   -- 32-bit unsigned integer

   type LWord is new Interfaces.Unsigned_64;
   -- 64-bit unsigned integer

   type SInt is new Interfaces.Integer_8;
   --  8-bit signed integer

   type Int is new Interfaces.Integer_16;
   -- 16-bit signed integer

   type DInt is new Interfaces.Integer_32;
   -- 32-bit signed integer

   type LInt is new Interfaces.Integer_64;
   -- 64-bit signed integer

   function SHL
     (Value  : Byte;
      Amount : Natural) return Byte renames Shift_Left;
   -- <summary>Shift Left Byte</summary>

   function SHR
     (Value  : Byte;
      Amount : Natural) return Byte renames Shift_Right;
   -- <summary>Shift Right Byte</summary>

   function SHL
     (Value  : Word;
      Amount : Natural) return Word renames Shift_Left;
   -- <summary>Shift Left Word</summary>

   function SHR
     (Value  : Word;
      Amount : Natural) return Word renames Shift_Right;
   -- <summary>Shift Right Word</summary>

   function SHL
     (Value  : DWord;
      Amount : Natural) return DWord renames Shift_Left;
   -- <summary>Shift Left DWord</summary>

   function SHR
     (Value  : DWord;
      Amount : Natural) return DWord renames Shift_Right;
   -- <summary>Shift Right DWord</summary>

   function ROL
     (Value  : Byte;
      Amount : Natural) return Byte renames Rotate_Left;
   -- <summary>Rotate Left Byte</summary>

   function ROR
     (Value  : Byte;
      Amount : Natural) return Byte renames Rotate_Right;
   -- <summary>Rotate Right Byte</summary>

   function ROL
     (Value  : Word;
      Amount : Natural) return Word renames Rotate_Left;
   -- <summary>Rotate Left Word</summary>

   function ROR
     (Value  : Word;
      Amount : Natural) return Word renames Rotate_Right;
   -- <summary>Rotate Right Word</summary>

   function ROL
     (Value  : DWord;
      Amount : Natural) return DWord renames Rotate_Left;
   -- <summary>Rotate Left DWord</summary>

   function ROR
     (Value  : DWord;
      Amount : Natural) return DWord renames Rotate_Right;
   -- <summary>Rotate Right DWord</summary>

   -- Global variables

   Quit : Boolean := False;
   -- <description>
   -- When Quit is set, the application shall exit.
   -- Quit is set either by SIGINT Handler or by GUI.
   -- All tasks should care about Quit and terminate themselves.
   -- </description>

   type Byte_Array is array (Integer range <>) of
     aliased Byte;
   for Byte_Array'Component_Size use Byte'Size;
   pragma Pack(Byte_Array);
   type Byte_Array_Access is access all Byte_Array;

   type Word_Array is array (Integer range <>) of
     aliased Word;
   for Word_Array'Component_Size use Word'Size;
   pragma Pack(Word_Array);
   type Word_Array_Access is access all Word_Array;

end A4A;
